//+------------------------------------------------------------------+
//|                                                  AccountInfo.mqh |
//|                      Copyright  2010, MetaQuotes Software Corp. |
//|                                       http://www.metaquotes.net/ |
//|                                              Revision 2010.02.08 |
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
//| Class CAccountInfo.                                              |
//| Appointment: Class for access to account info.                   |
//| Revision 2009.11.10                                              |
//+------------------------------------------------------------------+
class CAccountInfo
  {
public:
   //--- fast access methods to the integer account propertyes
   long              Login();
   ENUM_ACCOUNT_TRADE_MODE   TradeMode();
   string                    TradeModeDescription();
   long                      Leverage();
   ENUM_ACCOUNT_STOPOUT_MODE MarginMode();
   string                    MarginModeDescription();
   bool              TradeAllowed();
   bool              TradeExpert();
   //--- fast access methods to the double account propertyes
   double            Balance();
   double            Credit();
   double            Profit();
   double            Equity();
   double            Margin();
   double            FreeMargin();
   double            MarginLevel();
   double            MarginCall();
   double            MarginStopOut();
   //--- fast access methods to the string account propertyes
   string            Name();
   string            Server();
   string            Currency();
   string            Company();
   //--- access methods to the API MQL5 functions
   long              InfoInteger(ENUM_ACCOUNT_INFO_INTEGER prop_id);
   double            InfoDouble(ENUM_ACCOUNT_INFO_DOUBLE prop_id);
   string            InfoString(ENUM_ACCOUNT_INFO_STRING prop_id);
   double            FreeMarginCheck(const string symbol,ENUM_ORDER_TYPE trade_operation,double volume);
  };
//+------------------------------------------------------------------+
//| Get the property value "ACCOUNT_LOGIN".                          |
//| INPUT:  no.                                                      |
//| OUTPUT: the property value "ACCOUNT_LOGIN".                      |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
long CAccountInfo::Login()
  {
   return(AccountInfoInteger(ACCOUNT_LOGIN));
  }
//+------------------------------------------------------------------+
//| Get the property value "ACCOUNT_TRADE_MODE".                     |
//| INPUT:  no.                                                      |
//| OUTPUT: the property value "ACCOUNT_TRADE_MODE".                 |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
ENUM_ACCOUNT_TRADE_MODE CAccountInfo::TradeMode()
  {
   return((ENUM_ACCOUNT_TRADE_MODE)AccountInfoInteger(ACCOUNT_TRADE_MODE));
  }
//+------------------------------------------------------------------+
//| Get the property value "ACCOUNT_TRADE_MODE" as string.           |
//| INPUT:  no.                                                      |
//| OUTPUT: the property value "ACCOUNT_TRADE_MODE" as string.       |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
string CAccountInfo::TradeModeDescription()
  {
   string str;
//---
   switch(TradeMode())
     {
      case ACCOUNT_TRADE_MODE_DEMO:
         str="Demo trading account";
         break;
      case ACCOUNT_TRADE_MODE_CONTEST:
         str="Contest trading account";
         break;
      case ACCOUNT_TRADE_MODE_REAL:
         str="Real trading account";
         break;
      default: 
         str="Unknown trade account";
     }
//---
   return(str);
  }
//+------------------------------------------------------------------+
//| Get the property value "ACCOUNT_LEVERAGE".                       |
//| INPUT:  no.                                                      |
//| OUTPUT: the property value "ACCOUNT_LEVERAGE".                   |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
long CAccountInfo::Leverage()
  {
   return(AccountInfoInteger(ACCOUNT_LEVERAGE));
  }
//+------------------------------------------------------------------+
//| Get the property value "ACCOUNT_MARGIN_SO_MODE".                 |
//| INPUT:  no.                                                      |
//| OUTPUT: the property value "ACCOUNT_MARGIN_SO_MODE".             |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
ENUM_ACCOUNT_STOPOUT_MODE CAccountInfo::MarginMode()
  {
   return((ENUM_ACCOUNT_STOPOUT_MODE)AccountInfoInteger(ACCOUNT_MARGIN_SO_MODE));
  }
//+------------------------------------------------------------------+
//| Get the property value "ACCOUNT_MARGIN_SO_MODE" as string.       |
//| INPUT:  no.                                                      |
//| OUTPUT: the property value "ACCOUNT_MARGIN_SO_MODE" as string.   |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
string CAccountInfo::MarginModeDescription()
  {
   string str;
//---
   switch(MarginMode())
     {
      case ACCOUNT_STOPOUT_MODE_PERCENT:
         str="Level is specified in percentage";
         break;
      case ACCOUNT_STOPOUT_MODE_MONEY:
         str="Level is specified in money";
         break;
      default:
         str="Unknown margin mode";
     }
//---
   return(str);
  }
//+------------------------------------------------------------------+
//| Get the property value "ACCOUNT_TRADE_ALLOWED".                  |
//| INPUT:  no.                                                      |
//| OUTPUT: the property value "ACCOUNT_TRADE_ALLOWED".              |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CAccountInfo::TradeAllowed()
  {
   return((bool)AccountInfoInteger(ACCOUNT_TRADE_ALLOWED));
  }
//+------------------------------------------------------------------+
//| Get the property value "ACCOUNT_TRADE_EXPERT".                   |
//| INPUT:  no.                                                      |
//| OUTPUT: the property value "ACCOUNT_TRADE_EXPERT".               |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CAccountInfo::TradeExpert()
  {
   return((bool)AccountInfoInteger(ACCOUNT_TRADE_EXPERT));
  }
//+------------------------------------------------------------------+
//| Get the property value "ACCOUNT_BALANCE".                        |
//| INPUT:  no.                                                      |
//| OUTPUT: the property value "ACCOUNT_BALANCE".                    |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
double CAccountInfo::Balance()
  {
   return(AccountInfoDouble(ACCOUNT_BALANCE));
  }
//+------------------------------------------------------------------+
//| Get the property value "ACCOUNT_CREDIT".                         |
//| INPUT:  no.                                                      |
//| OUTPUT: the property value "ACCOUNT_CREDIT".                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
double CAccountInfo::Credit()
  {
   return(AccountInfoDouble(ACCOUNT_CREDIT));
  }
//+------------------------------------------------------------------+
//| Get the property value "ACCOUNT_PROFIT".                         |
//| INPUT:  no.                                                      |
//| OUTPUT: the property value "ACCOUNT_PROFIT".                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
double CAccountInfo::Profit()
  {
   return(AccountInfoDouble(ACCOUNT_PROFIT));
  }
//+------------------------------------------------------------------+
//| Get the property value "ACCOUNT_EQUITY".                         |
//| INPUT:  no.                                                      |
//| OUTPUT: the property value "ACCOUNT_EQUITY".                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
double CAccountInfo::Equity()
  {
   return(AccountInfoDouble(ACCOUNT_EQUITY));
  }
//+------------------------------------------------------------------+
//| Get the property value "ACCOUNT_MARGIN".                         |
//| INPUT:  no.                                                      |
//| OUTPUT: the property value "ACCOUNT_MARGIN".                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
double CAccountInfo::Margin()
  {
   return(AccountInfoDouble(ACCOUNT_MARGIN));
  }
//+------------------------------------------------------------------+
//| Get the property value "ACCOUNT_FREEMARGIN".                     |
//| INPUT:  no.                                                      |
//| OUTPUT: the property value "ACCOUNT_FREEMARGIN".                 |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
double CAccountInfo::FreeMargin()
  {
   return(AccountInfoDouble(ACCOUNT_FREEMARGIN));
  }
//+------------------------------------------------------------------+
//| Get the property value "ACCOUNT_MARGIN_LEVEL".                   |
//| INPUT:  no.                                                      |
//| OUTPUT: the property value "ACCOUNT_MARGIN_LEVEL".               |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
double CAccountInfo::MarginLevel()
  {
   return(AccountInfoDouble(ACCOUNT_MARGIN_LEVEL));
  }
//+------------------------------------------------------------------+
//| Get the property value "ACCOUNT_MARGIN_SO_CALL".                 |
//| INPUT:  no.                                                      |
//| OUTPUT: the property value "ACCOUNT_MARGIN_SO_CALL".             |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
double CAccountInfo::MarginCall()
  {
   return(AccountInfoDouble(ACCOUNT_MARGIN_SO_CALL));
  }
//+------------------------------------------------------------------+
//| Get the property value "ACCOUNT_MARGIN_SO_SO".                   |
//| INPUT:  no.                                                      |
//| OUTPUT: the property value "ACCOUNT_MARGIN_SO_SO".               |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
double CAccountInfo::MarginStopOut()
  {
   return(AccountInfoDouble(ACCOUNT_MARGIN_SO_SO));
  }
//+------------------------------------------------------------------+
//| Get the property value "ACCOUNT_NAME".                           |
//| INPUT:  no.                                                      |
//| OUTPUT: the property value "ACCOUNT_NAME".                       |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
string CAccountInfo::Name()
  {
   return(AccountInfoString(ACCOUNT_NAME));
  }
//+------------------------------------------------------------------+
//| Get the property value "ACCOUNT_SERVER".                         |
//| INPUT:  no.                                                      |
//| OUTPUT: the property value "ACCOUNT_SERVER".                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
string CAccountInfo::Server()
  {
   return(AccountInfoString(ACCOUNT_SERVER));
  }
//+------------------------------------------------------------------+
//| Get the property value "ACCOUNT_CURRENCY".                       |
//| INPUT:  no.                                                      |
//| OUTPUT: the property value "ACCOUNT_CURRENCY".                   |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
string CAccountInfo::Currency()
  {
   return(AccountInfoString(ACCOUNT_CURRENCY));
  }
//+------------------------------------------------------------------+
//| Get the property value "ACCOUNT_COMPANY".                        |
//| INPUT:  no.                                                      |
//| OUTPUT: the property value "ACCOUNT_COMPANY".                    |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
string CAccountInfo::Company()
  {
   return(AccountInfoString(ACCOUNT_COMPANY));
  }
//+------------------------------------------------------------------+
//| Access functions AccountInfoInteger(...).                        |
//| INPUT:  prop_id  -identifier integer properties.                 |
//| OUTPUT: the integer property value.                              |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
long CAccountInfo::InfoInteger(ENUM_ACCOUNT_INFO_INTEGER prop_id)
  {
   return(AccountInfoInteger(prop_id));
  }
//+------------------------------------------------------------------+
//| Access functions AccountInfoDouble(...).                         |
//| INPUT:  prop_id  -identifier double properties.                  |
//| OUTPUT: the double property value.                               |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
double CAccountInfo::InfoDouble(ENUM_ACCOUNT_INFO_DOUBLE prop_id)
  {
   return(AccountInfoDouble(prop_id));
  }
//+------------------------------------------------------------------+
//| Access functions AccountInfoString(...).                         |
//| INPUT:  prop_id  -identifier string properties.                  |
//| OUTPUT: the string property value.                               |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
string CAccountInfo::InfoString(ENUM_ACCOUNT_INFO_STRING prop_id)
  {
   return(AccountInfoString(prop_id));
  }
//+------------------------------------------------------------------+
//| Access functions AccountFreeMarginCheck(...).                    |
//| INPUT:  name            -symbol name,                            |
//|         trade_operation -trade operation,                        |
//|         volume          -volume of the opening position.         |
//| OUTPUT: the free margin value.                                   |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
double CAccountInfo::FreeMarginCheck(const string symbol,ENUM_ORDER_TYPE trade_operation,double volume)
  {
   double margin       =0.0;
   double free_margin  =FreeMargin();
   double contract_size=SymbolInfoDouble(symbol,SYMBOL_TRADE_CONTRACT_SIZE);
   long   leverage     =Leverage();
   string work;
   double bid;
//---
   switch(SymbolInfoInteger(symbol,SYMBOL_TRADE_CALC_MODE))
     {
      case SYMBOL_CALC_MODE_FOREX:
         work=SymbolInfoString(symbol,SYMBOL_CURRENCY_MARGIN)+Currency();
         bid=SymbolInfoDouble(work,SYMBOL_BID);
         margin=bid*volume*contract_size/leverage;
         margin=free_margin-margin;
         break;
      case SYMBOL_CALC_MODE_CFD:
         break;
      case SYMBOL_CALC_MODE_FUTURES:
         break;
      case SYMBOL_CALC_MODE_CFDINDEX:
         break;
      case SYMBOL_CALC_MODE_CFDLEVERAGE:
         break;
     }
//---
   return(margin);
  }
//+------------------------------------------------------------------+
